package eGuideDog::Helper;

use strict;
use base 'Exporter';
use eGuideDog::Speaker;
use eGuideDog::Basic;
use XML::Simple;

#our @EXPORT = qw(help);

sub new;
sub DESTROY;
sub help;

sub test;
#test();

sub test() {
  my $helper = eGuideDog::Helper->new();
  $helper->help('enter_eguidedog');
}

# constructor
# PARAMETER:
#   speaker: eGuideDog::Speaker object reference, undef for auto-generate a new one
# RETURN: eGuideDog::Helper object reference
sub new {
  shift; # class
  my $self = {};

  ($self->{rule_xml}, $self->{history_xml}, $self->{speaker}) = @_;
  $self->{rule_xml} = 'rule.xml' if (!$self->{rule_xml});
  $self->{history_xml} = 'history.xml' if (!$self->{history_xml});

  bless($self, __PACKAGE__);

  read_config($self);

  return $self;
}

sub DESTROY {
  my $self = shift;

  save_config($self);
}

sub read_config {
  my $self = shift;

  my $define = XMLin($self->{rule_xml});
  use Data::Dumper;
#  print Dumper($define);
  $self->{rule} = $define->{rule};
  $self->{frequency} = $define->{frequency};

  $self->{history} = XMLin($self->{history_xml}, ForceArray => ['show', 'enter']);
#  print Dumper($self->{history});
}

sub save_config {
  my $self = shift;

  my $hs = XMLout($self->{history}, RootName => 'histories');
  open(HIS_TMP, '>', "$self->{history_xml}.tmp") or warn;
  print HIS_TMP $hs;
  close(HIS_TMP) or warn;
  rename($self->{history_xml}, "$self->{history_xml}.old") or warn;
  rename("$self->{history_xml}.tmp", $self->{history_xml}) or warn;
}

# call for help
# PARAMETER:
#   self: object reference
#   status: the status you are in, which evoke proper help
# OUTPUT: help message in text or speech
sub help {
  my $self = shift;
  my $id = shift;

  # push entering history
  my $now = time;
  my $hour_ago = $now - 3600;
  my $day_ago = $now - 86400;
  my $week_ago = $now - 604800;
  my $month_ago = $now - 2592000;
  push(@{$self->{history}->{$id}->{enter}}, $now);

  my $fqc = $self->{frequency}->{$self->{rule}->{$id}->{frequency}};
  my $his = $self->{history}->{$id};
  warn("state $id not exists") if (!exists $self->{rule}->{$id});
  my $rule = $self->{rule}->{$id};
  my $show_his = $his->{show};
  $his->{total_enter}++;

  my $show = 1;
  $show = 0 if ($self->{rule}->{$id}->{disable});

  # re-count every hour
  if ($show &&(!$his->{update_time} || $his->{update_time} < $hour_ago)) {
    $his->{update_time} = $now;
    $his->{month_times} = 0;
    $his->{week_times} = 0;
    $his->{day_times} = 0;
    $his->{hour_times} = 0;

    # drop data older than a month
    while ($show_his->[0] && $show_his->[0] < $month_ago) {
      shift(@{$show_his});
    }

    foreach (@{$show_his}) {
      $his->{hour_times}++ if ($_ > $hour_ago);
      $his->{day_times}++ if ($_ > $day_ago);
      $his->{week_times}++ if ($_ > $week_ago);
      $his->{month_times}++;
    }
  }

  $show = 0 if ($show && $fqc->{max_times_per_hour} > 0 && $fqc->{max_times_per_hour} <= $his->{hour_times});
  $show = 0 if ($show && $fqc->{max_times_per_day} > 0 && $fqc->{max_times_per_day} <= $his->{day_times});
  $show = 0 if ($show && $fqc->{max_times_per_week} > 0 && $fqc->{max_times_per_week} <= $his->{week_times});
  $show = 0 if ($show && $fqc->{max_times_per_month} > 0 && $fqc->{max_times_per_month} <= $his->{month_times});

  if ($show) {
    push(@{$self->{history}->{$id}->{show}}, $now);
    $his->{total_show}++;
    if ($rule->{instant}) {
      if ($rule->{message}) {
	return $rule->{message};
      }
    } else {
      if ($rule->{message}) {
	return $rule->{message};
      }
      # add to watch list
    }
  }

  # initialized_eGuideDog
  my $status = $id;
  if ($status eq 'initialized_eGuideDog') {
    $self->{speaker}->speak(gettext('Welcome using eGuideDog.'));
  }

  # getting_key # when someting is 'ing', the helper should start a new thread to monitor the status and give help.
  elsif ($status eq 'getting_key') {
  }

  # quit_eGuideDog
  elsif ($status eq 'quit_eGuideDog') {
    $self->{speaker}->speak(gettext('Good bye!'));
  }

  # back_page
  elsif ($status eq 'back_page') {
    $self->{speaker}->speak(gettext('Navigating back.'));
  }

  # no_page_back
  elsif ($status eq 'no_page_back') {
    $self->{speaker}->speak(gettext('No more page to navigate back.'));
  }

  # execute_HTML_element
  elsif ($status eq 'execute_HTML_element') {
  }

  # go_previous_valid_node

  # no_more_content
  elsif ($status eq 'no_more_content') {
    $self->{speaker}->speak(gettext('No more contents.'));
  }

  # no_more_parts
  elsif ($status eq 'no_more_parts') {
    $self->{speaker}->speak(gettext('No more parts.'));
  }

  # go_next_valid_node

  # go_previous_part
  elsif ($status eq 'go_previous_part') {
  }

  # go_next_part

  # go_next_tab_item

  # stop_speech

  # continue_speech

  # change_voice_style

  # change_language

  # speak_again

  # speak_next_word

  # help
}

1;

__END__

=head1 NAME

eGuideDog::Helper - a system for showing help message.

=head1 SYNOPSIS

use eGuideDog::Helper;

my $john = eGuideDog::Helper->new();

$john->help('welcome');

=head2 METHODS

new();

help(status);

=head2 PROPERTIES


=head1 DESCRIPTION

a system for showing help message.


=head1 CHANGES

Feb 11, 2007 v 0.35

=head1 EXAMPLE


=head1 SEE ALSO


=head1 AUTHOR

Cameron Wong, e-guidedog.sf.net


=head1 COPYRIGHT AND LICENSE


Copyright (C) 2007 by Cameron Wong

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.6 or,
at your option, any later version of Perl 5 you may have available.

=cut
